

#include "clesperanto.hpp"

#include <algorithm>
#include <random>

template <class type>
auto
run_test(const std::array<size_t, 3> & shape, const cle::MemoryType & mem_type) -> bool
{
  std::vector<type> input1(shape[0] * shape[1] * shape[2]);
  std::vector<type> input2(shape[0] * shape[1] * shape[2]);
  std::vector<type> valid(shape[0] * shape[1] * shape[2]);

  static std::uniform_int_distribution<int> distribution(0, 1);
  static std::default_random_engine         generator;
  std::generate(input1.begin(), input1.end(), []() { return static_cast<type>(distribution(generator)); });
  std::generate(input2.begin(), input2.end(), []() { return static_cast<type>(distribution(generator)); });
  std::transform(input1.begin(), input1.end(), input2.begin(), valid.begin(), std::multiplies<type>());

  cle::Clesperanto cle;
  cle.GetDevice()->WaitForKernelToFinish();
  auto gpu_input1 = cle.Push<type>(input1, shape, mem_type);
  auto gpu_input2 = cle.Push<type>(input2, shape, mem_type);
  auto gpu_output = cle.Create<type>(shape, mem_type);
  cle.BinaryAnd(gpu_input1, gpu_input2, gpu_output);
  auto output = cle.Pull<type>(gpu_output);

  return std::equal(output.begin(), output.end(), valid.begin());
}

auto
main(int argc, char ** argv) -> int
{
  if (!run_test<float>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 10, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 10, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 10, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 10, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 10, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 10, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 10, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 10, 5, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 10, 5, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 10, 5, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 10, 5, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 10, 5, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 10, 5, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 10, 5, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 10, 1, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 10, 5, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 10, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 10, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 10, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 10, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 10, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 10, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 10, 5, 3 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 10, 5, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 10, 5, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 10, 5, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 10, 5, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 10, 5, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 10, 5, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  return EXIT_SUCCESS;
}