

#include "clesperanto.hpp"

#include <algorithm>
#include <random>

template <class type>
auto
run_test(const std::array<size_t, 3> & shape, const cle::MemoryType & mem_type) -> bool
{
  std::vector<type> input(shape[0] * shape[1] * shape[2]);
  std::vector<type> valid(shape[0] * shape[1] * shape[2]);
  std::fill(input.begin(), input.end(), static_cast<type>(0));
  std::fill(valid.begin(), valid.end(), static_cast<type>(0));
  const int center = (shape[0] / 2) + (shape[1] / 2) * shape[0] + (shape[2] / 2) * shape[1] * shape[0];
  input[center] = 100;
  if (valid.size() == 27)
    valid = { static_cast<type>(1.41674721F), static_cast<type>(2.33582091F), static_cast<type>(1.41674721F),
              static_cast<type>(2.33582115F), static_cast<type>(3.85111761F), static_cast<type>(2.33582115F),
              static_cast<type>(1.41674721F), static_cast<type>(2.33582091F), static_cast<type>(1.41674721F),
              static_cast<type>(2.33582115F), static_cast<type>(3.85111761F), static_cast<type>(2.33582115F),
              static_cast<type>(3.85111785F), static_cast<type>(6.34941959F), static_cast<type>(3.85111785F),
              static_cast<type>(2.33582115F), static_cast<type>(3.85111761F), static_cast<type>(2.33582115F),
              static_cast<type>(1.41674721F), static_cast<type>(2.33582091F), static_cast<type>(1.41674721F),
              static_cast<type>(2.33582115F), static_cast<type>(3.85111761F), static_cast<type>(2.33582115F),
              static_cast<type>(1.41674721F), static_cast<type>(2.33582091F), static_cast<type>(1.41674721F) };

  if (valid.size() == 9)
    valid = { static_cast<type>(5.85501814F), static_cast<type>(9.6532917F),  static_cast<type>(5.85501814F),
              static_cast<type>(9.65329266F), static_cast<type>(15.9155874F), static_cast<type>(9.65329266F),
              static_cast<type>(5.85501814F), static_cast<type>(9.6532917F),  static_cast<type>(5.85501814F) };

  if (valid.size() == 3)
    valid = { static_cast<type>(24.1971455F), static_cast<type>(39.8943443F), static_cast<type>(24.1971455F) };

  cle::Clesperanto cle;
  cle.GetDevice()->WaitForKernelToFinish();
  auto gpu_input = cle.Push<type>(input, shape, mem_type);
  auto gpu_output = cle.Create<type>(shape, mem_type);
  cle.GaussianBlur(gpu_input, gpu_output, 1, 1, 1);
  auto output = cle.Pull<type>(gpu_output);

  // ! how can we improve float accuracy between gpu and cpu?
  std::transform(output.begin(), output.end(), output.begin(), [](const type & x) {
    return static_cast<type>(std::round(static_cast<float>(x) * 10) / 10);
  });
  std::transform(valid.begin(), valid.end(), valid.begin(), [](const type & x) {
    return static_cast<type>(std::round(static_cast<float>(x) * 10) / 10);
  });

  return std::equal(output.begin(), output.end(), valid.begin());
}

auto
main(int argc, char ** argv) -> int
{
  if (!run_test<float>({ 3, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 3, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 3, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 3, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 3, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 3, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 3, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 3, 3, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 3, 3, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 3, 3, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 3, 3, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 3, 3, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 3, 3, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 3, 3, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 3, 3, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 3, 3, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 3, 3, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 3, 3, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 3, 3, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 3, 3, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 3, 3, 3 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 3, 1, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 3, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 3, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 3, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 3, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 3, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 3, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 3, 3, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 3, 3, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 3, 3, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 3, 3, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 3, 3, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 3, 3, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 3, 3, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 3, 3, 3 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 3, 3, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 3, 3, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 3, 3, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 3, 3, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 3, 3, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 3, 3, 3 }, cle::IMAGE)) return EXIT_FAILURE;
  return EXIT_SUCCESS;
}
