

#include <random>

#include "clesperanto.hpp"

template <class type>
auto
run_test(const std::array<size_t, 3> & shape, const cle::MemoryType & mem_type) -> bool
{
  std::vector<type> input(shape[0] * shape[1] * shape[2]);
  std::vector<type> valid(shape[0] * shape[1] * shape[2]);
  if (shape[2] > 1)
  {
    input = { 0, 0, 0, 0, 0, 0, 1, 2, 3, 0, 0, 2, 3, 4, 0, 0, 4, 4, 5, 0, 0, 0, 0, 0, 0,
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

    valid = { 0, 1, 2, 3, 0, 1, 1, 2, 3, 3, 2, 2, 3, 4, 4, 4, 4, 4, 5, 5, 0, 4, 4, 5, 0,
              0, 0, 0, 0, 0, 0, 1, 2, 3, 0, 0, 2, 3, 4, 0, 0, 4, 4, 5, 0, 0, 0, 0, 0, 0 };
  }
  else if (shape[1] > 1)
  {
    input = { 0, 0, 0, 0, 0, 0, 1, 2, 3, 0, 0, 2, 3, 4, 0, 0, 4, 4, 5, 0, 0, 0, 0, 0, 0 };
    valid = { 0, 1, 2, 3, 0, 1, 1, 2, 3, 3, 2, 2, 3, 4, 4, 4, 4, 4, 5, 5, 0, 4, 4, 5, 0 };
  }
  else
  {
    input = { 0, 2, 3, 3, 0 };
    valid = { 2, 2, 3, 3, 3 };
  }

  cle::Clesperanto cle;
  cle.GetDevice()->WaitForKernelToFinish();
  auto gpu_input = cle.Push<type>(input, shape, mem_type);
  auto gpu_flag = cle.Create<type>({ 1, 1, 1 }, mem_type);
  auto gpu_output = cle.Create<type>(shape, mem_type);
  cle.OnlyzeroOverwriteMaximumDiamond(gpu_input, gpu_flag, gpu_output);
  auto output = cle.Pull<type>(gpu_output);

  return std::equal(output.begin(), output.end(), valid.begin());
}

auto
main(int argc, char ** argv) -> int
{
  if (!run_test<float>({ 5, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 5, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 5, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 5, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 5, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 5, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 5, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 5, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 5, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 5, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 5, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 5, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 5, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 5, 5, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 5, 5, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 5, 5, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 5, 5, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 5, 5, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 5, 5, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 5, 5, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 5, 5, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 5, 1, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 5, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 5, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 5, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 5, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 5, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 5, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 5, 5, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 5, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 5, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 5, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 5, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 5, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 5, 5, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 5, 5, 2 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 5, 5, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 5, 5, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 5, 5, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 5, 5, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 5, 5, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 5, 5, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  return EXIT_SUCCESS;
}