

#include <random>

#include "clesperanto.hpp"

template <class type>
auto
run_test(const std::array<size_t, 3> & shape, const cle::MemoryType & mem_type) -> bool
{
  std::vector<type> input1(shape[0] * shape[1] * shape[2]);
  std::vector<type> input2(shape[0]);
  std::vector<type> valid(shape[0] * shape[1] * shape[2]);
  type              value = 0;
  for (auto it = input2.begin(); it != input2.end(); ++it)
  {
    *it = static_cast<type>((int)rand() % 10 + 10);
  }
  for (auto it = input1.begin(), it_valid = valid.begin(); (it != input1.end()) && (it_valid != valid.end());
       ++it, ++it_valid)
  {
    value = static_cast<type>((int)rand() % 10);
    *it = value;
    *it_valid = static_cast<type>(input2[value]);
  }

  cle::Clesperanto cle;
  cle.GetDevice()->WaitForKernelToFinish();
  auto gpu_input1 = cle.Push<type>(input1, shape, mem_type);
  auto gpu_input2 = cle.Push<type>(input2, { input2.size(), 1, 1 }, mem_type);
  auto gpu_output = cle.Create<type>(shape, mem_type);
  cle.ReplaceIntensities(gpu_input1, gpu_input2, gpu_output);
  auto output = cle.Pull<type>(gpu_output);

  return std::equal(output.begin(), output.end(), valid.begin());
}

auto
main(int argc, char ** argv) -> int
{
  if (!run_test<float>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 10, 7, 5 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 10, 7, 5 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 10, 7, 5 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 10, 7, 5 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 10, 7, 5 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 10, 7, 5 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 10, 7, 5 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 10, 1, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 10, 7, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 10, 7, 5 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 10, 7, 5 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 10, 7, 5 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 10, 7, 5 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 10, 7, 5 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 10, 7, 5 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 10, 7, 5 }, cle::IMAGE)) return EXIT_FAILURE;
  return EXIT_SUCCESS;
}