

#include <random>

#include "clesperanto.hpp"

template <class type>
auto
run_test(const std::array<size_t, 3> & shape, const cle::MemoryType & mem_type) -> bool
{
  std::vector<type> input(shape[0] * shape[1] * shape[2]);
  std::vector<type> valid(shape[0] * 1 * shape[2]);
  std::fill(input.begin(), input.end(), static_cast<type>(1));
  std::fill(valid.begin(), valid.end(), static_cast<type>(shape[1]));

  cle::Clesperanto cle;
  cle.GetDevice()->WaitForKernelToFinish();
  auto gpu_input = cle.Push<type>(input, shape, mem_type);
  auto gpu_output = cle.Create<type>({ shape[0], shape[2], 1 }, mem_type);
  cle.SumYProjection(gpu_input, gpu_output);
  auto output = cle.Pull<type>(gpu_output);

  return std::equal(output.begin(), output.end(), valid.begin());
}

auto
main(int argc, char ** argv) -> int
{
  if (!run_test<float>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 10, 1, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 10, 7, 1 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 5, 3, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<int>({ 5, 3, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned int>({ 5, 3, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<short>({ 5, 3, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned short>({ 5, 3, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<char>({ 5, 3, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<unsigned char>({ 5, 3, 2 }, cle::BUFFER))
    return EXIT_FAILURE;
  if (!run_test<float>({ 10, 1, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 10, 1, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 10, 7, 1 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 10, 7, 1 }, cle::IMAGE)) return EXIT_FAILURE;
  if (!run_test<float>({ 5, 3, 2 }, cle::IMAGE))
    return EXIT_FAILURE;
  //    if (!run_test<int>({ 5, 3, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned int>({ 5, 3, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<short>({ 5, 3, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned short>({ 5, 3, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<char>({ 5, 3, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  //    if (!run_test<unsigned char>({ 5, 3, 2 }, cle::IMAGE)) return EXIT_FAILURE;
  return EXIT_SUCCESS;
}